#include "laba.h"
#include <math.h>

/* Цвета */

#define YELLOW QColor(200, 200, 100)
#define YELLOW_BLIND QColor(200, 200, 100, 10)
#define YELLOW_LIGHT QColor(255, 255, 55)
#define GREEN QColor(50, 200, 0)
#define GREEN_BLIND QColor(50, 200, 0, 50)
#define RED QColor(255, 100, 50)

/* Функци-помошники */

#ifndef M_PI
#define M_PI 3.14
#endif // M_PI

#define RANDOM ( qrand() / (float) RAND_MAX )

#define SQR(X) ( (X) * (X) )

/* ============== */
/* Шаблон задачек */
/* ============== */

const QVector<MyElem> &Laba::data() const
{
	return _data;
}

/* ============== */
/* Задача номер 8 */
/* ============== */

Problem8::Problem8() { generate(); }

const char *Problem8::name() const { return "[8] Центр массы многоугольника"; }

void Problem8::generate()
{
	// Генерация точек (по кругу)
	QPolygonF poly;
	for (float alpha = 0; alpha < 2*M_PI; alpha += 2./9/5 * M_PI * (2*RANDOM+1)) {
		float rad = 30 * (10 * RANDOM + 1);
		poly << QPointF(rad * cos(alpha), rad * sin(alpha));
	}

	// Вычисляем центр массы
	QPointF massCenter(0, 0); // Сначала центр массы (0; 0)
	float s = 0; // Площать равна нулю
	QPointF mid(0, 0);
	QPointF prev = poly.last();
	foreach (QPointF next, poly) {
		QPointF v1 = prev - mid;
		QPointF v2 = next - mid;
		float sNew = 0.5 * fabs( v1.x() * v2.y() - v2.x() * v1.y() );
		massCenter += sNew * (mid + prev + next) / 3.;
		s += sNew;
		prev = next;
	}
	massCenter /= s;

	// Запоминаем вычисленные данные
	_data.clear(); // Стираем старые данные
	_data << MyElem(Loop, GREEN, poly); // Контур многоугольника
	_data << MyElem(Points, YELLOW, poly); // Вершины многоугольника
	_data << MyElem(Points, RED, QPolygonF() << massCenter); // Центр массы многоугольника
}

/* =============== */
/* Задача номер 17 */
/* =============== */

Problem17::Problem17() { generate(); }

const char *Problem17::name() const { return "[17] Точки в треугольнике"; }

void Problem17::generate()
{
	// Генерация точек и треугольника
	QPolygonF points;
	for (int n = 100 + qrand() % 200; n; n--)
		points << QPointF(500*(RANDOM-0.5), 500*(RANDOM-0.5));

	float alpha, rad;
	QPointF p1, p2, p3;
	alpha = 2 * M_PI * RANDOM;
	rad = 150 * (RANDOM + 1);
	p1 = QPointF(rad * cos(alpha), rad * sin(alpha));
	alpha += 2/6. * M_PI * (RANDOM+1);
	rad = 150 * (RANDOM + 1);
	p2 = QPointF(rad * cos(alpha), rad * sin(alpha));
	alpha += 2/6. * M_PI * (RANDOM+1);
	rad = 150 * (RANDOM + 1);
	p3 = QPointF(rad * cos(alpha), rad * sin(alpha));

	// Решение задачи
	QPolygonF pointsIn, pointsOut;
	foreach (QPointF p, points) {
		float d12 = (p1-p).x() * (p2-p).y() - (p2-p).x() * (p1-p).y();
		float d23 = (p2-p).x() * (p3-p).y() - (p3-p).x() * (p2-p).y();
		float d31 = (p3-p).x() * (p1-p).y() - (p1-p).x() * (p3-p).y();
		if ((d12 * d23 > 0) && (d23 * d31 > 0))
			pointsIn << p;
		else
			pointsOut << p;
	}

	// Запоминаем решение
	_data.clear(); // Стираем старые данные
	_data << MyElem(Loop, YELLOW, QPolygonF() << p1 << p2 << p3); // Треугольник
	_data << MyElem(Points, YELLOW_LIGHT, QPolygonF() << p1 << p2 << p3); // Вершины треугльника
	_data << MyElem(Points, GREEN, pointsOut); // Запминаем обычные точки
	_data << MyElem(Points, RED, pointsIn); // Внутренние точки
}

/* =============== */
/* Задача номер 28 */
/* =============== */

Problem28::Problem28() { generate(); }

const char *Problem28::name() const { return "[28] Минимальный периметр"; }

// Ключевая точка (для сортировки)
QPointF p0;

// Правило сортировки по полярному углу
bool polarSort(const QPointF &a, const QPointF &b) {
	float ax = a.x() - p0.x(), ay = a.y() - p0.y(), al = sqrt(SQR(ax) + SQR(ay)), cos2a = ax / al;
	float bx = b.x() - p0.x(), by = b.y() - p0.y(), bl = sqrt(SQR(bx) + SQR(by)), cos2b = bx / bl;
	return (cos2a > cos2b) || ((cos2a == cos2b) && (al < bl));
}

// Левый поворот?
bool leftTurn(const QPointF &a, const QPointF &b, const QPointF &c) {
	return (b.x() - a.x()) * (c.y() - a.y()) - (c.x() - a.x()) * (b.y() - a.y()) > 0;
}

void Problem28::generate()
{
	// Генерация точек (по кругу)
	QPolygonF points;
	for (int n = 10 + rand() % 101; n; n--) {
		float alpha = 2 * M_PI * RANDOM;
		float rad = 25 * (10 * RANDOM + 1);
		points << QPointF(rad * cos(alpha), rad * sin(alpha));
	}

	// Поиск самую левую из самых нижних точек
	QPolygonF::iterator i0 = points.begin();
	QPolygonF::iterator i = points.begin();
	while (++i != points.end()) {
		if ((i0->y() > i->y()) || ((i0->y() == i->y()) && (i0->x() > i->x())))
			i0 = i;
	}
	p0 = *i0;
	points.erase(i0);

	// Сортируем по полярному углу относительно прямой, исходящей из p0;
	qSort(points.begin(), points.end(), polarSort);

	// Ищем оболочку за O(n) шагов
	QPolygonF::iterator a, b;
	QPolygonF hull;
	hull << points.last() << p0;
	i = points.begin();
	while (i != points.end()) {
		b = hull.end() - 1;
		a = b - 1;
		if (leftTurn(*a, *b, *i)) {
			hull << *i;
			i++;
		} else {
			hull.pop_back();
		}
	}
	hull.pop_back();

	// Запоминаем решение
	_data.clear(); // Стираем старые данные
//	_data << MyElem(Line, YELLOW_BLIND, points);
	_data << MyElem(Points, GREEN, points);
	_data << MyElem(Loop, RED, hull);
	_data << MyElem(Points, YELLOW, hull);
	_data << MyElem(Points, RED, QPolygonF() << p0);
}
