#include "mygl.h"
#include "laba.h"
#include <QtGui>

MyGL::MyGL(QWidget *parent) : QGLWidget(QGLFormat(QGL::SampleBuffers), parent)
{
	// Параметры позиционирования и масштаба
	center = QPointF(0, 0);
	zoom = 1;

	// Цвета и шрифты
	background = QBrush(QColor(64, 32, 64));
	textPen = QPen(Qt::white);
	textFont.setPixelSize(20);

	// Лабораторки здесь
	laba = NULL;
}

void MyGL::paintEvent(QPaintEvent *event)
{
	// Начинаем рисование
	painter.begin(this);

	// Устанавливаем сглаживание
	painter.setRenderHint(QPainter::Antialiasing);

	// Заливаем фон
	painter.fillRect(event->rect(), background);

	if (laba != NULL) // Если лаба есть, то рисуем ее
	{
		// Сохраняем состояние до зума с перемещением
		painter.save();

		// Устанавливаем параметры вида
		painter.translate(event->rect().center() + zoom*center);
		painter.scale(zoom, zoom);

		// Рисуем лабу
		QVector<MyElem> data = laba->data();
		foreach (MyElem elem, data) {
			QPen pen(elem.color);
			pen.setCapStyle(Qt::RoundCap);
			pen.setJoinStyle(Qt::RoundJoin);
			switch (elem.type) {
				case Points :
					pen.setWidth(5);
					painter.setPen(pen);
					painter.drawPoints(elem.polygon);
					break;

				case Line :
					pen.setWidth(2);
					painter.setPen(pen);
					painter.drawPolyline(elem.polygon);
					break;

				case Loop :
					pen.setWidth(2);
					painter.setPen(pen);
					painter.drawPolygon(elem.polygon);
					break;
				default :
						qDebug("Warning - 'None' state!");
			}
		}

		// Восстанавливаем сохраненное состояние
		painter.restore();

		// Пишем текст
		painter.setPen(textPen);
		painter.setFont(textFont);
		painter.drawText(event->rect().topLeft()+QPoint(10,25), tr(laba->name()));
	}
	else // Если лабы нет, то просим выбрать лабу
	{
		painter.setPen(textPen);
		painter.setFont(textFont);
		painter.drawText(event->rect().topLeft()+QPoint(10,25), tr("Выбирай лабу скорей!"));
	}

	// Конец рисования
	painter.end();
}

void MyGL::setZoom(float z)
{
	zoom = z;
}

/* Работа с мышью */

QPoint oldPos;

void MyGL::mousePressEvent(QMouseEvent *event)
{
	if (event->buttons() & Qt::LeftButton) {
		qDebug("Mouse left key pressed");
		oldPos = event->pos();
	}
	QGLWidget::mousePressEvent(event);
}

void MyGL::mouseMoveEvent(QMouseEvent *event)
{
	if (event->buttons() & Qt::LeftButton) {
		qDebug("Mouse left key moved");
		center += (event->pos() - oldPos) / zoom;
		oldPos = event->pos();
	}
	QGLWidget::mouseMoveEvent(event);
	this->update();
}
